<?php
namespace FiloBlu\Storelocator\Controller\Index;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem;
use Magento\Framework\App\Request\DataPersistorInterface;

class Appointment extends Action
{
    private $dataPersistor;
    /**
     * @return \Magento\Framework\Controller\Result\Redirect|\Magento\Framework\View\Result\Page
     */

    protected $context;
    private $fileSystem;
    protected $_scopeConfig;
    protected $storesFactory;


    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;



    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */


    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     */

    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        Filesystem $fileSystem,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $_storeManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    )
    {
        parent::__construct($context);
        $this->fileSystem = $fileSystem;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $_storeManager;
        $this->_scopeConfig = $scopeConfig;
        $this->storesFactory = $storesFactory;
        $this->_resultJsonFactory = $resultJsonFactory;
    }

    public function execute()
    {

        $resultJson = $this->_resultJsonFactory->create();
        $post = $this->getRequest()->getPostValue();


        $shopId = $this->getRequest()->getParam('id');
        $shop = $this->storesFactory->create()->load($shopId);
        $shopData = $shop->getdata();
        $shopDataArray = $shopData;


        $storeName = $this->_scopeConfig->getValue(
            'trans_email/ident_support/name',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        $storeEmail = $this->_scopeConfig->getValue(
            'trans_email/ident_support/email',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        // Email sender to store

        $fromEmailToStore = $post['appointment_email'];
        $fromNameToStore = $post['appointment_name'];

        $templateVarsToStore = [
            'store' => $this->_storeManager->getStore(),
            'customer_name' => $post['appointment_name'],
            'customer_lastname' => $post['appointment_lastname'],
            'customer_phone' => $post['appointment_phone'],
            'customer_email' => $post['appointment_email'],
            'chosen_date' => $post['appointment_date'],
            'time_slot' => $post['appointment_time_slot'],
            'appointment_message' => $post['appointment_message'],
            'appointment_hours' => $post['appointment_hours'],
            'product_name' => $post['product_name'],
            'product_sku' => $post['product_sku'],
            'product_price' => $post['product_price'],
            'product_image' => $post['product_image'],
            'wishlist_products' => $post['wishlist_products']
        ];

        $fromToStore = ['email' => $fromEmailToStore, 'name' => $fromNameToStore];
        $toToStore = $storeEmail;


        $templateOptions = [
            'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
            'store' => \Magento\Store\Model\Store::DEFAULT_STORE_ID
        ];

        try {
            $transport = $this->_transportBuilder->setTemplateIdentifier('appointment_template')
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVarsToStore)
                ->setFrom($fromToStore)
                ->addTo($toToStore)
                ->getTransport();
            $transport->sendMessage();

        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $resultJson->setData(['success' => false, 'error' => true, 'message' => __('There was a problem with the request: %1', $e->getMessage())]);
        } catch (\Exception $e) {
            return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
        }

        //Email sender to customer

        $templateVarsToCustomer = [
            'store' => $this->_storeManager->getStore(),
            'customer_name' => $post['appointment_name'],
            'store_name' => $shopDataArray['title'],
            'store_address' => $shopDataArray['street'],
            'store_postcode' => $shopDataArray['postcode'],
            'store_city' => $shopDataArray['city'],
            'store_phone' => $shopDataArray['phone_1'],
            'store_email' => $shopDataArray['email'],
            'product_name' => $post['product_name'],
            'product_sku' => $post['product_sku'],
            'product_price' => $post['product_price'],
            'product_image' => $post['product_image'],
            'wishlist_products' => $post['wishlist_products']
        ];

        $fromEmailToCustomer = $storeEmail;
        $fromNameToCustomer = $storeName;

        $fromToCustomer = ['email' => $fromEmailToCustomer, 'name' => $fromNameToCustomer];

        $toToCustomer = $post['appointment_email'];

        try {
            $transport = $this->_transportBuilder->setTemplateIdentifier('appointment_template_to_customer')
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVarsToCustomer)
                ->setFrom($fromToCustomer)
                ->addTo($toToCustomer)
                ->getTransport();
            $transport->sendMessage();

        }
            catch (\Exception $e) {
                return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
        }

        $resultJson->setData(['success' => true, 'error' => false, 'message' => __('Apointment request recived, a recap email has been sent to you')]);

        return $resultJson;
    }
}